#include <raylib.h>
#include "preferred_size.h"
#include <math.h>
#include <raymath.h>

void DrawTextBox(const char *text, int x, int y, int width, int height, float alignX, float alignY)
{
  DrawRectangle(x, y, width, height, WHITE);
  DrawRectangleLinesEx((Rectangle){x, y, width, height}, 2.0f, BLACK);
  Font font = GetFontDefault();
  float fontSize = font.baseSize * 2;
  float spacing = 2;
  Vector2 textSize = MeasureTextEx(font, text, fontSize, spacing);
  float textX = x + (width - textSize.x) * alignX;
  float textY = y + (height - textSize.y) * alignY;
  Vector2 pos = {textX, textY};
  DrawTextEx(font, text, pos, fontSize, spacing, BLACK);
}

int main(void)
{
  int screenWidth = 600, screenHeight = 350;
  GetPreferredSize(&screenWidth, &screenHeight);
  InitWindow(screenWidth, screenHeight, "Loading models with multiple meshes");

  Camera camera = {0};
  camera.position = (Vector3){8.0f, 7.0f, 5.0f};
  camera.target = (Vector3){0.0f, 1.0f, 0.0f};
  camera.up = (Vector3){0.0f, 1.0f, 0.0f};
  camera.fovy = 45.0f;
  camera.projection = CAMERA_PERSPECTIVE;

  Model model = LoadModel("data/quadset.glb");

  Vector3 position = {0.0f, 0.0f, 0.0f};

  SetTargetFPS(30);

  while (!WindowShouldClose())
  {
  if (IsPaused())
    {
      // canvas is not visible in browser - do nothing
      continue;
    }
    
    if (IsMouseButtonDown(MOUSE_LEFT_BUTTON))
      UpdateCamera(&camera, CAMERA_FIRST_PERSON);

    BeginDrawing();
    ClearBackground(GRAY);
    DrawRectangleGradientV(0, 0, GetScreenWidth(), GetScreenHeight(), SKYBLUE, LIGHTGRAY);

    BeginMode3D(camera);
    DrawModel(model, position, 1.0f, WHITE);
    for (int i = 0; i < model.meshCount; i++)
    {
      BoundingBox box = GetMeshBoundingBox(model.meshes[i]);
      DrawBoundingBox(box, RED);
    }
    DrawGrid(10, 1.0f);
    EndMode3D();

    DrawTextBox("GLTF loading multiple meshes", GetScreenWidth() / 2 - 180, 10, 360, 40, 0.5f, 0.5f);
    EndDrawing();
  }

  UnloadModel(model);
  CloseWindow();

  return 0;
}